<?php

namespace Aedes\EveryPay\Controller\Checkout;

class Result extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Aedes\EveryPay\Helper\ResponseHandler
     */
    protected $responseHandler;

    /**
     * @var \Aedes\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Aedes\EveryPay\Helper\ResponseHandler $responseHandler
     * @param \Aedes\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Aedes\EveryPay\Helper\ResponseHandler $responseHandler,
        \Aedes\EveryPay\Logger\Logger $logger
    ) {
        parent::__construct($context);
        $this->responseHandler = $responseHandler;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function execute()
    {
        $response = $this->getRequest()->getParams();
        $result = null;

        try {
            $result = $this->responseHandler->handleResponse($response);
        } catch (\Exception $e) {
            $this->logger->error('Exception while processing response', [$e->getMessage(), $e->getTrace()]);
        }

        if (isset($response['payment_state'])) {
            if ($response['payment_state'] == 'settled' || $response['payment_state'] == 'authorized') {
                if ($result === true) {
                    return $this->_redirect('checkout/onepage/success');
                } else if (is_array($result)) {
                    foreach ($result as $error) {
                        $this->messageManager->addErrorMessage($error);
                    }

                    $this->logger->error('Error while validating response payload', $result);
                }
            }
        }

        $this->messageManager->addNoticeMessage(
            __('It seems that something went wrong. Please don\'t hesitate to contact us if any errors occurred during the payment process')
        );

        return $this->_redirect('checkout/onepage/failure');
    }
}