<?php

namespace Aedes\EveryPay\Helper;

use Magento\Framework\Exception\LocalizedException;

class OrderProcessor extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\DB\TransactionFactory
     */
    protected $transactionFactory;

    /**
     * @var \Magento\Sales\Api\Data\OrderInterface
     */
    protected $orderInterface;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\DB\TransactionFactory $transactionFactory
     * @param \Magento\Sales\Api\Data\OrderInterface $orderInterface
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\DB\TransactionFactory $transactionFactory,
        \Magento\Sales\Api\Data\OrderInterface $orderInterface
    ) {
        $this->transactionFactory = $transactionFactory;
        $this->orderInterface = $orderInterface;

        parent::__construct($context);
    }

    /**
     * @param array $data
     * @return \Magento\Sales\Model\Order
     * @throws LocalizedException
     * @throws bool
     */
    public function process($data)
    {
        /** @var $order \Magento\Sales\Model\Order */
        $order = $this->orderInterface->loadByIncrementId($data['order_reference']);

        if (!($data['payment_state'] == 'settled' || $data['payment_state'] == 'authorized')) {
            return $order;
        }

        //Ignore duplicate processing, since we are dealing with full captures
        if ($order->hasInvoices()) {
            return $order;
        }

        $order->getPayment()->setTransactionId($data['payment_reference']);

        /** @var $invoice \Magento\Sales\Model\Order\Invoice */
        $invoice = $order->prepareInvoice();

        if (!$invoice) {
            throw new LocalizedException(__('Invoice could not be created.'));
        }

        $invoice->register();
        $invoice->capture();

        $order = $invoice->getOrder();
        $payment = $order->getPayment();
        $transaction = $payment->getCreatedTransaction();

        $this->transactionFactory->create()
            ->addObject($payment)
            ->addObject($transaction)
            ->addObject($invoice)
            ->addObject($order)
            ->save();

        return $order;
    }
}
