<?php

namespace Aedes\EveryPay\Model;

use Magento\Framework\Exception\LocalizedException;

class GuestPaymentInformationAdapter implements \Aedes\EveryPay\Api\GuestPaymentInformationAdapterInterface
{
    /**
     * @var \Magento\Checkout\Api\GuestPaymentInformationManagementInterface
     */
    protected $guestPaymentInformationManagement;

    /**
     * @var \Aedes\EveryPay\Model\Form\FormDataBuilder
     */
    protected $formDataBuilder;

    /**
     * @var \Aedes\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $guestPaymentInformationManagement
     * @param \Aedes\EveryPay\Model\Form\FormDataBuilder $formDataBuilder
     * @param \Aedes\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $guestPaymentInformationManagement,
        \Aedes\EveryPay\Model\Form\FormDataBuilder $formDataBuilder,
        \Aedes\EveryPay\Logger\Logger $logger
    ) {
        $this->guestPaymentInformationManagement = $guestPaymentInformationManagement;
        $this->formDataBuilder = $formDataBuilder;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function adaptPlaceOrder(
        $cartId,
        $email,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        try {
            $orderId = $this->guestPaymentInformationManagement->savePaymentInformationAndPlaceOrder(
                $cartId, $email, $paymentMethod, $billingAddress
            );

            return json_encode($this->formDataBuilder->getFields($orderId));
        } catch (\Exception $e) {
            $this->logger->error('An error occured while placing the order', [$e->getMessage(), $e->getTrace()]);
            throw new LocalizedException(__($e->getMessage()));
        }
    }
}
