<?php

namespace Aedes\EveryPay\Model;

use Magento\Framework\Exception\LocalizedException;

class PaymentInformationAdapter implements \Aedes\EveryPay\Api\PaymentInformationAdapterInterface
{
    /**
     * @var \Magento\Checkout\Api\PaymentInformationManagementInterface
     */
    protected $paymentInformationManagement;

    /**
     * @var \Aedes\EveryPay\Model\Form\FormDataBuilder
     */
    protected $formDataBuilder;

    /**
     * @var \Aedes\EveryPay\Model\Token\TokenManagement
     */
    protected $tokenManager;

    /**
     * @var \Aedes\EveryPay\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Checkout\Api\PaymentInformationManagementInterface $paymentInformationManagement
     * @param \Aedes\EveryPay\Model\Form\FormDataBuilder $formDataBuilder
     * @param \Aedes\EveryPay\Model\Token\TokenManagement $tokenManager
     * @param \Aedes\EveryPay\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Checkout\Api\PaymentInformationManagementInterface $paymentInformationManagement,
        \Aedes\EveryPay\Model\Form\FormDataBuilder $formDataBuilder,
        \Aedes\EveryPay\Model\Token\TokenManagement $tokenManager,
        \Aedes\EveryPay\Logger\Logger $logger
    ) {
        $this->paymentInformationManagement = $paymentInformationManagement;
        $this->formDataBuilder = $formDataBuilder;
        $this->tokenManager = $tokenManager;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function adaptPlaceOrder(
        $cartId,
        $tokenAgreement,
        $tokenId,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        try {
            $orderId = $this->paymentInformationManagement->savePaymentInformationAndPlaceOrder(
                $cartId, $paymentMethod, $billingAddress
            );

            $tokenData = $this->tokenManager->getTokenData($tokenAgreement, $tokenId);
            return json_encode($this->formDataBuilder->getFields($orderId, $tokenData));
        } catch (\Exception $e) {
            $this->logger->error('An error occured while placing the order', [$e->getMessage(), $e->getTrace()]);
            throw new LocalizedException(__($e->getMessage()));
        }
    }
}
