<?php

namespace Aedes\EveryPay\Model\Token;

class TokenManagement
{
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var \Aedes\EveryPay\Api\Data\TokenInterfaceFactory
     */
    protected $tokenFactory;

    /**
     * @var \Aedes\EveryPay\Api\Data\TokenRepositoryInterface
     */
    protected $tokenRepository;

    /**
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder
     * @param \Magento\Framework\Api\FilterBuilder $filterBuilder
     * @param \Aedes\EveryPay\Api\Data\TokenInterfaceFactory $tokenFactory
     * @param \Aedes\EveryPay\Api\Data\TokenRepositoryInterface $tokenRepository
     */
    public function __construct(
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\Api\SearchCriteriaBuilder $searchCriteriaBuilder,
        \Magento\Framework\Api\FilterBuilder $filterBuilder,
        \Aedes\EveryPay\Api\Data\TokenInterfaceFactory $tokenFactory,
        \Aedes\EveryPay\Api\Data\TokenRepositoryInterface $tokenRepository
    ) {
        $this->customerSession = $customerSession;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterBuilder = $filterBuilder;
        $this->tokenFactory = $tokenFactory;
        $this->tokenRepository = $tokenRepository;
    }

    /**
     * @param array $data
     * @return $this
     */
    public function saveToken($data)
    {
        if (!isset($data['cc_token']) || !$this->customerSession->getId()) {
            return $this;
        }

        //don't save duplicates
        //TODO create an UNIQUE constraint on db level and refactor this
        $customerFilter = $this->getCustomerFilter();
        $tokenFilter = $this->getTokenFilter($data['cc_token']);
        $this->searchCriteriaBuilder->addFilters([$customerFilter]);
        $this->searchCriteriaBuilder->addFilters([$tokenFilter]);
        $count = $this->tokenRepository->getList($this->searchCriteriaBuilder->create())->getTotalCount();

        if ($count == 0) {
            /** @var \Aedes\EveryPay\Api\Data\TokenInterface $tokenModel */
            $tokenModel = $this->tokenFactory->create();

            $tokenModel
                ->setCustomerId($this->customerSession->getId())
                ->setCcToken($data['cc_token'])
                ->setCcLastFourDigits($data['cc_last_four_digits'])
                ->setCcMonth($data['cc_month'])
                ->setCcYear($data['cc_year'])
                ->setCcHolderName($data['cc_holder_name'])
                ->setCcType($data['cc_type']);

            $this->tokenRepository->save($tokenModel);
        }

        return $this;
    }

    /**
     * @return \Magento\Framework\Api\ExtensibleDataInterface[]
     */
    public function getListByCustomerSession()
    {
        $this->searchCriteriaBuilder->addFilters([$this->getCustomerFilter()]);
        return $this->tokenRepository->getList($this->searchCriteriaBuilder->create())->getItems();
    }

    /**
     * @return \Magento\Framework\Api\Filter
     */
    protected function getCustomerFilter()
    {
        return $this->filterBuilder
            ->setField('customer_id')
            ->setValue($this->customerSession->getId())
            ->setConditionType('eq')
            ->create();
    }

    /**
     * @param string $token
     * @return \Magento\Framework\Api\Filter
     */
    protected function getTokenFilter($token)
    {
        return $this->filterBuilder
            ->setField('cc_token')
            ->setValue($token)
            ->setConditionType('eq')
            ->create();
    }

    /**
     * Security check whether the token belongs to the current customer
     *
     * @param \Aedes\EveryPay\Api\Data\TokenInterface $token
     * @return bool
     */
    public function isCustomersToken(\Aedes\EveryPay\Api\Data\TokenInterface $token)
    {
        return $token->getCustomerId() == $this->customerSession->getId();
    }

    /**
     * @param int|bool $isAgree
     * @param int $tokenId
     * @return array|null
     */
    public function getTokenData($isAgree, $tokenId)
    {
        if ($isAgree) {
            $tokenData = [
                'request_cc_token' => $isAgree,
                'token_id' => $tokenId
            ];
        } else {
            $tokenData = null;
        }

        return $tokenData;
    }
}